package com.diagbot.facade.data;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.AMedicalRecordContentDTO;
import com.diagbot.entity.MedicalRecordContent;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.MedicalRecordContentServiceImpl;
import com.diagbot.util.*;
import com.diagbot.vo.data.ADeleteFlagVO;
import com.diagbot.vo.data.AMedicalRecordContentVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class AMedicalRecordContentFacade extends MedicalRecordContentServiceImpl{

    @Qualifier("medicalRecordContentServiceImpl")
    @Autowired
    private MedicalRecordContentServiceImpl medicalRecordContentService;

    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    @Autowired
    private SqlProperties sqlProperties;

    private TZDBConn tzDBConn = new TZDBConn();

    /**
     * 终末质控-同步前一天的病历详细记录
     */
    public void executeMrRecordContent() {
        //String sql="select * from mr_mrcontent  where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        //String sql="select BLJLID,BLJLNR from mr_mrcontent  where cjcxrq>=(select CONVERT(varchar,GETDATE()-2,23)) and cjcxrq<(select CONVERT(varchar,GETDATE(),23)) ORDER BY cjcxrq DESC";
//        String sql = sqlProperties.getRecordContentSql();
        String sql = "select * from ai_v_sendmrrecording where behospitalcode in(select behospitalcode from AI_V_SENDPATIENTINFO where leavehospitaldate >=to_date('2020-06-01','yyyy-mm-dd') and leavehospitaldate<to_date('2020-06-15','yyyy-mm-dd'))";
        List<MedicalRecordContent> mrContentList = tzDBConn.getMedicalRecordContent(sql);
        execute(mrContentList);
    }
    /**
     * 数据引擎-通过接口更新病历内容
     * @param list
     * @return
     */
    public RespDTO<List<AMedicalRecordContentDTO>> executeMrRecordContent(List<AMedicalRecordContentVO> list) {
        try {
            if(list!=null && list.size()>0) {
                List<MedicalRecordContent> mrContentList=BeanUtil.listCopyTo(list,MedicalRecordContent.class);
                List<AMedicalRecordContentDTO> mrContentDtoList=BeanUtil.listCopyTo(list,AMedicalRecordContentDTO.class);
                execute(mrContentList);
                return RespDTO.onSuc(mrContentDtoList);
            }else{
                aMedAbnormalInfoFacade.saveAbnormalInfo("文书内容","",JSON.toJSONString(list),"","未接收到数据!");
                return RespDTO.onError("未接收到数据!");
            }
        }catch (Exception e){
            aMedAbnormalInfoFacade.saveAbnormalInfo("文书内容","",JSON.toJSONString(list),"",e.getMessage());
            return RespDTO.onError(e.getMessage());
        }

    }

    public void execute(List<MedicalRecordContent> mrContentList){
        try {
            List<MedicalRecordContent> addE = Lists.newLinkedList();
            List<MedicalRecordContent> updateE = Lists.newLinkedList();
            if (mrContentList != null && mrContentList.size() > 0) {
                mrContentList.stream().forEach(s -> {
                    QueryWrapper<MedicalRecordContent> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("rec_id", s.getRecId());
                    queryWrapper.eq("hospital_id", s.getHospitalId());
                    MedicalRecordContent mrContent = this.getOne(queryWrapper, false);
                    if (mrContent != null) {
                        s.setGmtModified(new Date());
                        //updateE.add(s);
                        medicalRecordContentService.update(s,queryWrapper);
                    } else {
                        s.setGmtCreate(new Date());
                        addE.add(s);
                    }
                });
            }
            if(addE.size()>0){
                medicalRecordContentService.saveBatch(addE);
            }
            /*if(updateE.size()>0){
                medicalRecordContentService.updateBatchByKey(updateE);
            }*/
            //aMedAbnormalInfoFacade.saveAbnormalInfo("文书内容",JSON.toJSONString(mrContentList),"","操作成功!");
        }catch (Exception e){
            aMedAbnormalInfoFacade.saveAbnormalInfo("文书内容","",JSON.toJSONString(mrContentList),"",e.getMessage());
        }
    }

    public RespDTO deleteFlag(ADeleteFlagVO aDeleteFlagVO){
        try {
            //验证数据是否存在
            if (StringUtil.isEmpty(aDeleteFlagVO.getRecId())) {
                return RespDTO.onError("请输入文书序号!");
            } else if (aDeleteFlagVO.getHospitalId() == null) {
                return RespDTO.onError("请输入医院编码!");
            }else {
                UpdateWrapper<MedicalRecordContent> updateWrapper=new UpdateWrapper<>();
                updateWrapper.eq("rec_id", aDeleteFlagVO.getRecId())
                        .eq("hospital_id", aDeleteFlagVO.getHospitalId())
                        .eq("is_deleted",IsDeleteEnum.N)
                        .set("is_deleted",IsDeleteEnum.Y)
                        .set("gmt_modified", DateUtil.now());

                Boolean flag=update(new MedicalRecordContent(),updateWrapper);
                //aMedAbnormalInfoFacade.saveAbnormalInfo("文书内容",JSON.toJSONString(aDeleteFlagVO),JSON.toJSONString(RespDTO.onSuc(flag)),"操作成功!");
                return RespDTO.onSuc(flag);
            }
        }catch (Exception e){
            aMedAbnormalInfoFacade.saveAbnormalInfo("文书内容",aDeleteFlagVO.getRecId(), JSON.toJSONString(aDeleteFlagVO),"",e.getMessage());
            return RespDTO.onError(e.getMessage());
        }
    }
}


