package com.diagbot.util;

import ca.uhn.hl7v2.model.Message;
import ca.uhn.hl7v2.parser.PipeParser;
import ca.uhn.hl7v2.util.Terser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

@Service
public class SocketServer {
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());
    //    @Value("${port}")
    private Integer port;
    private boolean started;
    private ServerSocket serverSocket;
    //使用多线程，需要线程池，防止并发过高时创建过多线程耗尽资源
    private ExecutorService threadPool = Executors.newCachedThreadPool();
    @Autowired
    private YWAnalysis ywAnalysis;

    public void start() {
        start(8091);
    }

    private void start(Integer port) {
        try {
            serverSocket = new ServerSocket(port == null ? this.port : port);
            started = true;
            logger.info("Socket服务已启动，占用端口： {}", serverSocket.getLocalPort());
        } catch (IOException e) {
            logger.error("端口异常信息", e);
            System.exit(0);
        }
        while (started) {
            try {
                Socket socket = serverSocket.accept();
                Runnable runnable = () -> {
                    try {
                        //接收客户端数据
                        StringBuilder socketReceiveMsg = onMessage(socket);
                        String receiveMsg = String.valueOf(socketReceiveMsg).replaceAll("\n", "\r").replace("\u000B", "").replace("\u001C", "").replace("\\\r","");
                        //处理逻辑
                        logger.info("receiveMsg========================>");
                        System.out.println(receiveMsg);
                        //返回给his消息格式
                        StringBuilder responseMsg = getResponseMsg(receiveMsg);
                        logger.info("responseMsg========================>");
                        System.out.println(responseMsg);
                        sendMessage(socket, String.valueOf(responseMsg));
                        ywAnalysis.executeAnalysis(receiveMsg);
                        //socket.close();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }finally {
                        try {
                            socket.shutdownInput();
                            socket.shutdownOutput();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                };
                //接收线程返回结果
                Future future = threadPool.submit(runnable);
                logger.info(future.isDone() + "--------");
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 接收客户端消息
     *
     * @param socket
     * @return
     */
    private static StringBuilder onMessage(Socket socket) {
        byte[] bytes = new byte[10240];
        int len;
        try {
            // 建立好连接后，从socket中获取输入流，并建立缓冲区进行读取
            InputStream inputStream = socket.getInputStream();
            StringBuilder sb = new StringBuilder();
            sb.append(new String(bytes, 0, inputStream.read(bytes), "UTF-8"));
//            while ((len = inputStream.read(bytes)) != -1) {
//
//                // 注意指定编码格式，发送方和接收方一定要统一，建议使用UTF-8
//                sb.append(new String(bytes, 0, len, "UTF-8"));
//            }
            //此处，需要关闭服务器的输出流，但不能使用inputStream.close().
            return sb;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 发送消息
     *
     * @param socket
     * @param message
     */
    private static void sendMessage(Socket socket, String message) {
        try {
            //向客户端返回数据
            OutputStream outputStream = socket.getOutputStream();
            //首先需要计算得知消息的长度
            byte[] sendBytes = message.getBytes("UTF-8");
            outputStream.write(sendBytes);
            outputStream.flush();
            //outputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 生成返回客户端消息
     *
     * @param receiveMsg
     * @return
     */
    private StringBuilder getResponseMsg(String receiveMsg) {
        StringBuilder responseMsg = new StringBuilder();
        try {
            PipeParser pipeParser = PipeParser.getInstanceWithNoValidation();
            Message message = pipeParser.parse(receiveMsg);
            Terser terser = new Terser(message);
            String sUid = UUID.randomUUID().toString();
            responseMsg.append("\u000B").append("MSH|")
                    .append(terser.get("/." + "MSH-2") + "|")
                    .append(terser.get("/." + "MSH-3") + "|")
                    .append(terser.get("/." + "MSH-4") + "|")
                    .append(terser.get("/." + "MSH-5") + "|")
                    .append(terser.get("/." + "MSH-6") + "|")
                    .append(YWDateUtils.getCurrentTime() + "||")
                    .append("ACK|" + sUid + "|")
                    .append(terser.get("/." + "MSH-11") + "|")
                    .append(terser.get("/." + "MSH-12"))
                    .append("\r\n")
                    .append("MSA|AA|")
                    .append(terser.get("/." + "MSH-10")).append("\u001C");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return responseMsg;
    }

}
