package com.diagbot.util;

import ca.uhn.hl7v2.model.Message;
import ca.uhn.hl7v2.parser.PipeParser;
import ca.uhn.hl7v2.util.Terser;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.entity.*;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.data.DoctorAdviceStatusEnum;
import com.diagbot.facade.BehospitalInfoFacade;
import com.diagbot.facade.data.*;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Component
public class YWAnalysis {

    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;
    @Autowired
    private ABehospitalInfoFacade aBehospitalInfoFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;
    @Autowired
    private ADoctorAdviceFacade aDoctorAdviceFacade;
    @Autowired
    private AMedPacsInfoFacade aMedPacsInfoFacade;
    @Autowired
    private AMedPacsResultFacade aMedPacsResultFacade;
    @Autowired
    private AMedLisInfoFacade aMedLisInfoFacade;
    @Autowired
    private AMedLisResultFacade aMedLisResultFacade;
    @Autowired
    private AMedCrisisFacade aMedCrisisFacade;
    @Autowired
    private AMedicalRecordFacade aMedicalRecordFacade;
    @Autowired
    private AMedicalRecordContentFacade aMedicalRecordContentFacade;
    @Autowired
    private AMedNurseFacade aMedNurseFacade;

    private TZDBConn tzDBConn = new TZDBConn();

    private PipeParser pipeParser = PipeParser.getInstanceWithNoValidation();

    public static final Long HOSPITAL_ID = Long.valueOf("5");//1:长兴，2：邵逸夫，3：台州市立医院，5义乌妇幼医院

    @Transactional
    public void executeAnalysis(String hisMessage) throws Exception{
        try {
            Message message = pipeParser.parse(hisMessage);
            Terser terser = new Terser(message);
            //入院/入科更新入院登记
            if (message.getName().equals("ADT_A01") || message.getName().equals("ADT_A09")) {
                aBehospitalInfoFacade.execute(getBehospitalInfoToView(terser.get("/.PV1-19-1")));
            }
            if (message.getName().equals("ADT_A16") ) {
                //预出院消息更新出院时间
                BehospitalInfo behospitalInfo = behospitalInfoFacade.getOne(new QueryWrapper<BehospitalInfo>()
                        .eq("behospital_code", terser.get("/.PV1-19-1"))
                        .eq("hospital_id", HOSPITAL_ID)
                        .eq("is_deleted", IsDeleteEnum.N), false);
                if(behospitalInfo != null && StringUtil.isNotEmpty(terser.get("/.PV1-45-1"))){
                    UpdateWrapper<BehospitalInfo> behospitalInfoUpdateWrapper = new UpdateWrapper<>();
                    behospitalInfoUpdateWrapper.eq("behospital_code", behospitalInfo.getBehospitalCode())
                    .eq("hospital_id", HOSPITAL_ID)
                    .eq("is_deleted", IsDeleteEnum.N)
                    .set("leave_hospital_date",DateUtil.parseDate(YWDateUtils.convJsonDateToString(terser.get("/.PV1-45-1"), DateUtil.DATE_TIME_FORMAT),"yyyy-MM-dd HH:mm:ss"))
                    .set("gmt_modified",DateUtil.now());
                    behospitalInfoFacade.update(behospitalInfoUpdateWrapper);
                }
            }else if (message.getName().equals("OMP_O09")) {
                //医嘱
                aDoctorAdviceFacade.execute(getDoctorAdviceToView(terser.get("/.ORC-2-1")));
            }else if (message.getName().equals("ORU_R01")) {
                //检查结果
                aMedPacsInfoFacade.execute(getPacsInfoToMsg(hisMessage));
                aMedPacsResultFacade.execute(getPacsResultToView(terser.get("/.ORC-2-1")));
            }else if (message.getName().equals("OUL_R21")) {
                //化验
                aMedLisInfoFacade.execute(getLisInfoToMsg(hisMessage));
                aMedLisResultFacade.execute(getLisResultToView(terser.get("/.OBR-2-3")));
            }else if (message.getName().equals("RAS_O17") && terser.get("/.ORDER/RXR-1").equals("3")) {
                //护理
                aMedNurseFacade.execute(getMedNurseToMsg(hisMessage));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 解析入院登记信息(视图获取/消息解析)
     *
     * @param queryParam
     * @return
     */
    public List<BehospitalInfo> getBehospitalInfoToView(String queryParam) {
        String queryLisResultSql = "SELECT * FROM AI_V_SENDPATIENTINFO t WHERE t.BEHOSPITALCODE='" + queryParam + "'";
        List<BehospitalInfo> behospitalInfoList = tzDBConn.getBehospitalInfo(queryLisResultSql);
        return behospitalInfoList;
    }

    public List<BehospitalInfo> getBehospitalInfoToMsg(String hisMessage) {
        List<BehospitalInfo> behospitalInfoList = Lists.newLinkedList();
        BehospitalInfo behospitalInfo = new BehospitalInfo();
        try {
            Message message = pipeParser.parse(hisMessage);
            Terser terser = new Terser(message);
            behospitalInfo.setBehospitalCode(terser.get("/.PV1-19-1"));
            behospitalInfo.setHospitalId(HOSPITAL_ID);
            behospitalInfo.setName(terser.get("/.PID-5-2"));
            String sex = terser.get("/.PID-8");
            if(sex.equals("F")){
                sex = "女";
            }else if(sex.equals("M")){
                sex = "男";
            }else if(sex.equals("O")){
                sex = "其他";
            }
            behospitalInfo.setSex(sex);
            if(StringUtil.isNotEmpty(terser.get("/.PID-7"))){
                behospitalInfo.setBirthday(DateUtil.parseDate(YWDateUtils.convJsonDateToString(terser.get("/.PID-7"), DateUtil.DATE_FORMAT)));
            }
            behospitalInfo.setFileCode(terser.get("/.PID-3(1)-1"));
            behospitalInfo.setWardCode(terser.get("/.PV1-3-1"));
            behospitalInfo.setWardName(terser.get("/.PV1-3-6"));
            behospitalInfo.setBehDeptId(terser.get("/.PV1-3-4-1"));
            behospitalInfo.setBehDeptName(terser.get("/.PV1-3-4-2"));
            //behospitalInfo.setBedCode();
            //behospitalInfo.setBedName();
            behospitalInfo.setInsuranceName(terser.get("/.PV1-18"));//医保类别待转
            //behospitalInfo.setJobType();
            if(StringUtil.isNotEmpty(terser.get("/.PV1-44-1"))){
                behospitalInfo.setBehospitalDate(DateUtil.parseDate(YWDateUtils.convJsonDateToString(terser.get("/.PV1-44-1"), DateUtil.DATE_TIME_FORMAT)));
            }
            //behospitalInfo.setLeaveHospitalDate();
            //behospitalInfo.setDiagnoseIcd();
            //behospitalInfo.setDiagnose();
            //behospitalInfo.setBehDoctorId();
            //behospitalInfo.setBehDoctorName();
            behospitalInfo.setDoctorId(terser.get("/.PV1-7-1"));
            behospitalInfo.setDoctorName(terser.get("/.PV1-7-3"));
            //behospitalInfo.setDirectorDoctorId();
            //behospitalInfo.setDirectorDoctorName();
            behospitalInfo.setIsPlacefile("0");
            //behospitalInfo.setPlacefileDate();
        } catch (Exception e) {
            e.printStackTrace();
            aMedAbnormalInfoFacade.saveAbnormalInfo("socketMsg病人住院登记","", JSON.toJSONString(hisMessage), "", e.getMessage());
        }
        behospitalInfoList.add(behospitalInfo);
        return behospitalInfoList;
    }

    /**
     * 解析医嘱(视图获取/消息解析)
     *
     * @param hisMessage
     * @return
     */
    public List<DoctorAdvice> getDoctorAdviceToMsg(String hisMessage) {
        List<DoctorAdvice> doctorAdviceList = Lists.newLinkedList();
        DoctorAdvice doctorAdviceVO = new DoctorAdvice();
        try {
            Message message = pipeParser.parse(hisMessage);
            Terser terser = new Terser(message);
            doctorAdviceVO.setDoctorAdviceId(terser.get("/.ORC-2-1"));
            doctorAdviceVO.setHospitalId(HOSPITAL_ID);
            doctorAdviceVO.setBehospitalCode(terser.get("/.PV1-19-1"));
            doctorAdviceVO.setOrderDoctorName(terser.get("/.ORC-12-3"));
            doctorAdviceVO.setFrequency(terser.get("/.ORC-7-2-1"));
//            doctorAdviceVO.setParentTypeId("");
            doctorAdviceVO.setDoctorAdviceType(terser.get("/.ORC-16-2"));//待更改
            doctorAdviceVO.setUsageNum(terser.get("/.ORC-7-1-1"));
            doctorAdviceVO.setUsageUnit(terser.get("/.ORC-7-6"));
            doctorAdviceVO.setDose(terser.get("/.RXO-11"));
            doctorAdviceVO.setDoseUnit(terser.get("/.RXO-12-2"));
            doctorAdviceVO.setMedModeType(terser.get("/.RXR-1-1"));
//            doctorAdviceVO.setMedicineType("");//待定

            doctorAdviceVO.setDaFrequency(terser.get("/.ORC-7-2-1"));
//            doctorAdviceVO.setDaDealType("");
            if (StringUtils.isNotEmpty(terser.get("/.ORC-7-4-1"))) {
                doctorAdviceVO.setDaStartDate(DateUtil.parseDate(YWDateUtils.convJsonDateToString(terser.get("/.ORC-7-4-1"), DateUtil.DATE_TIME_FORMAT), DateUtil.DATE_TIME_FORMAT));
            }
            doctorAdviceVO.setDaItemName(terser.get("/.RXO-1-2"));
            doctorAdviceVO.setDaStatus(DoctorAdviceStatusEnum.getName(terser.get("/.ORC-5")));
            if (StringUtils.isNotEmpty(terser.get("/.ORC-7-5-1"))) {
                doctorAdviceVO.setDaStopDate(DateUtil.parseDate(YWDateUtils.convJsonDateToString(terser.get("/.ORC-7-5-1"), DateUtil.DATE_TIME_FORMAT), DateUtil.DATE_TIME_FORMAT));
            }
            doctorAdviceVO.setDaGroupNo(terser.get("/.ORC-4-1"));
            doctorAdviceVO.setDaPrescriptionType(terser.get("/.ORC-16-2"));
//            doctorAdviceVO.setDaMedType("");待定
            doctorAdviceVO.setDoctorNotice(terser.get("/.RXO-7-2"));
            doctorAdviceVO.setDoctorId(terser.get("/.ORC-12-1"));
            doctorAdviceVO.setDoctorName(terser.get("/.ORC-12-3"));
            if (terser.get("/.ORC-5").equals("CA") || terser.get("/.ORC-5").equals("HD")) {
                doctorAdviceVO.setIsDeleted("Y");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        doctorAdviceList.add(doctorAdviceVO);
        return doctorAdviceList;
    }

    public List<DoctorAdvice> getDoctorAdviceToView(String queryParam) {
        String queryDoctorAdviceSql = "SELECT * FROM AI_V_SENDDOCTORADVICE t WHERE t.DOCTORADVICEID='" + queryParam + "'";
        List<DoctorAdvice> doctorAdviceList = tzDBConn.getDoctorAdvice(queryDoctorAdviceSql);
        return doctorAdviceList;
    }


    /**
     * 解析检查信息消息pacsInfo
     *
     * @param hisMessage
     * @return
     */
    public List<MedPacsInfo> getPacsInfoToMsg(String hisMessage) {
        List<MedPacsInfo> medPacsInfoList = Lists.newLinkedList();
        MedPacsInfo medPacsInfo = new MedPacsInfo();
        try {
            Message message = pipeParser.parse(hisMessage);
            Terser terser = new Terser(message);
            medPacsInfo.setRepNo(terser.get("/.ORC-2-1"));//报告单号
            medPacsInfo.setHospitalId(HOSPITAL_ID);
            medPacsInfo.setBehospitalCode(terser.get("/.PV1-19-1"));//住院病人ID
            medPacsInfo.setRepName(terser.get("/.OBR-15-1-2"));//报告名称
            medPacsInfo.setRepType(null);//报告类型代码
            medPacsInfo.setRepName(terser.get("/.OBR-4-5"));//检查类型名称
            medPacsInfo.setPart(terser.get("/.OBR-4-2"));//检查部位
            if (StringUtils.isNotEmpty(terser.get("/.OBR-22-1"))) {
                //发布时间
                medPacsInfo.setRepDate(DateUtil.formatDateTime(DateUtil.parseDate(YWDateUtils.convJsonDateToString(terser.get("/.OBR-22-1"), DateUtil.DATE_TIME_FORMAT), DateUtil.DATE_TIME_FORMAT)));
            }
            if (StringUtils.isNotEmpty(terser.get("/.ORC-9-1"))) {
                //报告检查时间
                medPacsInfo.setCheckDate(DateUtil.formatDateTime(DateUtil.parseDate(YWDateUtils.convJsonDateToString(terser.get("/.ORC-9-1"), DateUtil.DATE_TIME_FORMAT), DateUtil.DATE_TIME_FORMAT)));
            }
            medPacsInfo.setApplyDoctor(terser.get("/.ORC-12-3"));//申请医生
            medPacsInfo.setDeptId(terser.get("/.ORC-17-1"));//开单科室代码
            medPacsInfo.setDeptName(terser.get("/.ORC-17-2"));//开单科室名称
            medPacsInfo.setRegisterFlag(terser.get("/.OBR-25"));//报告单状态
            medPacsInfo.setReportDoctor(terser.get("/.ORC-12-3"));//报告医生
            medPacsInfo.setChecker(terser.get("/.ORC-11-3"));//报告审核医生
            medPacsInfo.setYjpdz(null);//云胶片地址
        } catch (Exception e) {
            e.printStackTrace();
        }
        medPacsInfoList.add(medPacsInfo);
        return medPacsInfoList;
    }

    /**
     * 解析检查结果消息(视图获取/消息解析)
     *
     * @param queryParam
     * @return
     */
    public List<MedPacsResult> getPacsResultToView(String queryParam) {
        String queryLisResultSql = "SELECT * FROM AI_V_SENDPACSRESULTS t WHERE t.REPTNO='" + queryParam + "'";
        List<MedPacsResult> medPacsResultList = tzDBConn.getPacsResult(queryLisResultSql);
        return medPacsResultList;
    }

    public List<MedPacsResult> getPacsResultToMsg(String hisMessage) {
        List<MedPacsResult> medPacsResultList = Lists.newLinkedList();
        MedPacsResult medPacsResult = new MedPacsResult();
        try {
            Message message = pipeParser.parse(hisMessage);
            Terser terser = new Terser(message);
            medPacsResult.setRepNo(terser.get("/.ORC-2-1"));
            medPacsResult.setHospitalId(HOSPITAL_ID);
            medPacsResult.setBehospitalCode(terser.get("/.PV1-19-1"));
            medPacsResult.setReptName(terser.get("/.OBR-4-5"));
            medPacsResult.setPart(terser.get("/.OBR-4-2"));
            if (terser.get("/.OBR-3-1").equals("IMP") && terser.get("/.OBR-2").equals("CE")) {
                medPacsResult.setClinicSight(terser.get("/.OBX-5"));
            }
            if (terser.get("/.OBR-3-1").equals("GDT") && terser.get("/.OBR-2").equals("TX")) {
                medPacsResult.setReptDiag(terser.get("/.OBX-5"));
            }
            medPacsResult.setRemark(null);
            medPacsResult.setClinicDiag(terser.get("/.DG1-4"));
            medPacsResult.setApplyDoctor(terser.get("/.ORC-12-1"));
            medPacsResult.setCheckerDoctor(terser.get("/.OBR-34-1-1"));
            medPacsResult.setReportDoctor(terser.get("/.ORC-10-1"));
            medPacsResult.setDeptName(terser.get("/.ORC-17-2"));
        } catch (Exception e) {
            e.printStackTrace();
        }
        medPacsResultList.add(medPacsResult);
        return medPacsResultList;
    }

    /**
     * 解析化验信息消息lisInfo
     *
     * @param hisMessage
     * @return
     */
    public List<MedLisInfo> getLisInfoToMsg(String hisMessage) {
        List<MedLisInfo> medLisInfoList = Lists.newLinkedList();
        MedLisInfo medLisInfo = new MedLisInfo();
        try {
            Message message = pipeParser.parse(hisMessage);
            Terser terser = new Terser(message);
            medLisInfo.setRepNo(terser.get("/.OBR-2-3"));//报告单号
            medLisInfo.setHospitalId(HOSPITAL_ID);
            medLisInfo.setBehospitalCode(terser.get("/.PV1-19-1"));//住院病人ID
            medLisInfo.setRepType(terser.get("/.OBR-4-4"));//报告类型代码
            medLisInfo.setRepName(terser.get("/.OBR-4-2"));//报告名称
            medLisInfo.setRepTypeName(null);//检查类型名称
            medLisInfo.setSlideType(null);//标本名称
            if (StringUtils.isNotEmpty(terser.get("/.OBR-22-1"))) {
                //发布时间
                medLisInfo.setRepDate(DateUtil.formatDateTime(DateUtil.parseDate(YWDateUtils.convJsonDateToString(terser.get("/.OBR-22-1"), DateUtil.DATE_TIME_FORMAT), DateUtil.DATE_TIME_FORMAT)));
            }
            if (StringUtils.isNotEmpty(terser.get("/.OBR-7-1"))) {
                //报告检查时间
                medLisInfo.setCheckDate(DateUtil.formatDateTime(DateUtil.parseDate(YWDateUtils.convJsonDateToString(terser.get("/.OBR-7-1"), DateUtil.DATE_TIME_FORMAT), DateUtil.DATE_TIME_FORMAT)));
            }
            medLisInfo.setApplyDoctor(terser.get("/.ORC-12-3"));//申请医生
            medLisInfo.setDeptId(terser.get("/.ORC-17-1"));//开单科室代码
            medLisInfo.setRegisterFlag(terser.get("/.OBR-25"));//报告单状态
        } catch (Exception e) {
            e.printStackTrace();
        }
        medLisInfoList.add(medLisInfo);
        return medLisInfoList;
    }

    /**
     * 化验结果消息(视图获取/消息解析)
     *
     * @param queryParam
     * @return
     */
    public List<MedLisResult> getLisResultToView(String queryParam) {
        String queryLisResultSql = "SELECT * FROM AI_V_SENDLISRESULTS t WHERE t.REPTNO='" + queryParam + "'";
        List<MedLisResult> medLisResultList = tzDBConn.getLisResult(queryLisResultSql);
        return medLisResultList;
    }

    public List<MedLisResult> getLisResultToMsg(String hisMessage) {
        List<MedLisResult> medLisResultList = Lists.newLinkedList();
        try {
            Message message = pipeParser.parse(hisMessage);
            Terser terser = new Terser(message);
            int OBXCounts = hisMsgHeaderCounts(hisMessage, "OBX|");
            for (int i = 0; i < OBXCounts; i++) {
                MedLisResult medLisResult = new MedLisResult();
                medLisResult.setRepNo(terser.get("/.OBR-2-3"));
                medLisResult.setHospitalId(HOSPITAL_ID);
                medLisResult.setBehospitalCode(terser.get("/.PV1-19-1"));
                medLisResult.setItemCode(terser.get("/.OBSERVATION(" + i + ")/OBX-3-1"));
                medLisResult.setItemName(terser.get("/.OBSERVATION(" + i + ")/OBX-3-2"));
                medLisResult.setResult(terser.get("/.OBSERVATION(" + i + ")/OBX-5"));
                //medLisResult.setReferenceMin("");
                //medLisResult.setReferenceMax("");
                //medLisResult.setUnit("");
                //medLisResult.setAbnormal("");
                //medLisResult.setColor("");
                medLisResultList.add(medLisResult);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return medLisResultList;
    }

    /**
     * 解析危急值结果(视图获取/消息解析)
     *
     * @param queryParam
     * @return
     */
    public List<MedCrisisInfo> getMedCrisisInfoToView(String queryParam) {
        String queryLisResultSql = "SELECT * FROM AI_V_SENDCRISIS t WHERE t.REPTNO='" + queryParam + "'";
        List<MedCrisisInfo> medCrisisInfoList = tzDBConn.getMedCrisisInfo(queryLisResultSql);
        return medCrisisInfoList;
    }

    /**
     * 消息解析护理
     *
     * @param hisMessage
     * @return
     */
    public List<MedNurse> getMedNurseToMsg(String hisMessage) {
        List<MedNurse> medNurseList = Lists.newLinkedList();
        try {
            Message message = pipeParser.parse(hisMessage);
            Terser terser = new Terser(message);
            int OBXCount = hisMsgHeaderCounts(hisMessage,"OBX|");
            for (int i = 0; i < OBXCount; i++) {
                MedNurse medNurse = new MedNurse();
                medNurse.setRepNo(terser.get("/.PV1-19-1")+"_"+terser.get("/.PID-3(1)-1")+"_"+i);
                medNurse.setHospitalId(HOSPITAL_ID);
                medNurse.setBehospitalCode(terser.get("/.PV1-19-1"));
                if (StringUtils.isNotEmpty(terser.get("/.OBSERVATION(" + i + ")/OBX-14-1"))) {
                    //测量时间
                    medNurse.setMeasureTime(DateUtil.parseDate(YWDateUtils.convJsonDateToString(terser.get("/.OBX-14-1"), DateUtil.DATE_TIME_FORMAT), DateUtil.DATE_TIME_FORMAT));
                }
                medNurse.setItemName(terser.get("/.OBSERVATION(" + i + ")/OBX-3-2"));
                medNurse.setItemValue(terser.get("/.OBSERVATION(" + i + ")/OBX-5"));
                medNurse.setItemUnit(terser.get("/.OBSERVATION(" + i + ")/OBX-6-1"));
                medNurse.setItemType(null);
                medNurseList.add(medNurse);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return medNurseList;
    }

    /**
     * 消息段循环
     */
    public int hisMsgHeaderCounts(String s, String ch) {
        //定义计数
        int count = 0;
        //判断出现次数
        for (int i = 0; i < s.length() - 1; i++) {
            if (s.indexOf(ch, i) >= 0) {
                count++;
                i = s.indexOf(ch, i) + ch.length() - 1;
            }
        }
        //返回次数
        return count;
    }
}
